pico-8 cartridge // http://www.pico-8.com
version 23
__lua__
-- mouselock demo
-- by ultrabrite

poke(0x5f2d, 1) -- enable mouse

mouse = {
	x=stat(32), y=stat(33), 	-- position
	dx=0, dy=0,	-- deltas

	lockable=false,
	locked=false,
	moved=false,

	update = function(m)
		local peep = peek(0x5f80)
		m.lockable = (peep>0)	-- proper html but maybe unlocked
		m.locked   = (peep==2)	-- pointer locked within html
		m.dx,m.dy  = 0,0
		if (m.locked) then -- locked mouse, deltas available
			if peek(0x5f81)!=0 then -- new data ready
				-- get mouse deltas from gpio
				m.dx,m.dy = peek2(0x5f82)/256,peek2(0x5f84)/256
				-- update mouse pos info
				m.x,m.y = mid(0,m.x + m.dx,127),mid(0,m.y + m.dy,127)
				 -- ready for more!
				poke(0x5f81,0)
			end
		else -- fallback on standard mouse coords
			local ox,oy = m.x,m.y
			-- get mouse pos
			m.x,m.y = stat(32),stat(33)
			-- calculate sensible deltas
			if m.x&127==m.x and ox&127==ox and m.y&127==m.y and oy&127==oy then
				-- binary pico screen border still returns pointer coords
				-- albeit <0 or >127, so we know we're offscreen
				-- so we only provide delta if we are -and were- onscreen
				-- this smoothes things a little during dev
				m.dx,m.dy = m.x-ox,m.y-oy
			end
		end
		-- standard mouse buttons & wheel
		m.lmb = stat(34)&1!=0
		m.rmb = stat(34)&2!=0
		m.mmb = stat(34)&4!=0
		m.wheel = stat(36)

		m.moved = m.dx!=0 or m.dy!=0
		return m.moved
	end,
}

-->8
-- demo
_sin,sin=sin,function(a) return -_sin(a) end --can't get used to it

--2 shades of green from extended palette
pal(9,138,1)
pal(10,139,1)

function vadds(a,b, s)
	return {x=a.x+s*b.x, y=a.y+s*b.y, z=a.z+s*b.z}
end

eye = {
	pos = {x=0,y=0,z=2},
	yaw=0,
	pitch=0,
	ycos=1,
	ysin=0,
	pcos=1,
	psin=0,

	input=function(m)
		local spd = 0.2
		local rev = 0.01

		if (btn(0)) then m.pos = vadds(m.pos, m.vx, -spd) end
		if (btn(1)) then m.pos = vadds(m.pos, m.vx, spd) end
		if (btn(2)) then m.pos = vadds(m.pos, m.vy, spd) end
		if (btn(3)) then m.pos = vadds(m.pos, m.vy, -spd) end


		if (not btn(2) and not btn(3)) then m.pos.z-=0.1 end

		if (m.pos.z<1) m.pos.z=1
	end,

	mouselook = function(m)
		if mouse.moved then
			m.yaw += mouse.dx/128
			m.pitch = mid(-0.125,m.pitch-mouse.dy/128,0.25)
		end
	end,

	update = function(m)
		m:input()
		m:mouselook()

		ycos,ysin = cos(m.yaw),sin(m.yaw)
		pcos,psin = cos(m.pitch),sin(m.pitch)

		m.vx = { x= ycos,      y= ysin,      z= 0    }
		m.vy = { x=-ysin*pcos, y= ycos*pcos, z=-psin }
		m.vz = { x=-ysin*psin, y= ycos*psin, z= pcos }
	end,
}

function draw_ground(eye)
	poke(0x5F38, 2) -- texture map 2x2
	poke(0x5F39, 2)

	for i=-64,63 do

		local vu = vadds(eye.vy, eye.vz, -i/64) --scanline dir

		if (vu.z>0) then --sky -- todo:some texturing too?
			line(-64,i,64,i,12)
		else -- ground
			local vl = vadds(vu,eye.vx,-1)	-- left ray
			local vr = vadds(vu,eye.vx, 1)	-- right ray

			local kl = -eye.pos.z/vl.z
			local kr = -eye.pos.z/vr.z

			local tx = eye.pos.x + vl.x * kl
			local ty = eye.pos.y + vl.y * kl

			local ux = eye.pos.x + vr.x * kr
			local uy = eye.pos.y + vr.y * kr

			tline(-64,i,64,i, tx%2,ty%2, (ux-tx)/128,(uy-ty)/128)
		end
	end
end


function _init()
	camera(-64,-64)
end

function _update()
	mouse:update()
	eye:update()
end

function printc(s,a,b,c)
	print(s,a-#s*2,b,c)
end
function _draw()
	draw_ground(eye)
	circ(0,0,2,0)

	circ(mouse.x-64,mouse.y-64,1,8)
	--print(stat(1),-64,-64,0)
	--if (mouse.lockable) then
		printc(mouse.locked and "mouse locked" or "mouse unlocked",0,-63,0)
	--end
end

__gfx__
00000000bbbbbbbbaaaaaaaa0000000088888888333333330000000000000000bbbbbbbb9999999900000000bbbbbbbb99999999000000000000000000000000
00000000bbbbbbbbaaaaaaaa0000000088888888333333330000000000000000bbbbbbbb9999999900000000bbbbbbbb99999999000000000000000000000000
00700700bbbbbbbbaaaaaaaa0077770088888888333333330000000000000000bbbbbbbb9999999900000000bbbbbbbb99999999000000000000000000000000
00077000bbbbbbbbaaaaaaaa0077770088888888333333330000000000000000bbbbbbbb9999999900000000bbbbbbbb99999999000000000000000000000000
00077000bbbbbbbbaaaaaaaa0077770088888888333333330000000000000000bbbbbbbb9999999900000000bbbbbbbb99999999000000000000000000000000
00700700bbbbbbbbaaaaaaaa0077770088888888333333330000000000000000bbbbbbbb9999999900000000bbbbbbbb99999999000000000000000000000000
00000000bbbbbbbbaaaaaaaa0000000088888888333333330000000000000000bbbbbbbb9999999900000000bbbbbbbb99999999000000000000000000000000
00000000bbbbbbbbaaaaaaaa0000000088888888333333330000000000000000bbbbbbbb9999999900000000bbbbbbbb99999999000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000033333333aaaaaaaa0000000033333333aaaaaaaa000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000033333333aaaaaaaa0000000033333333aaaaaaaa000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000033333333aaaaaaaa0000000033333333aaaaaaaa000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000033333333aaaaaaaa0000000033333333aaaaaaaa000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000033333333aaaaaaaa0000000033333333aaaaaaaa000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000033333333aaaaaaaa0000000033333333aaaaaaaa000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000033333333aaaaaaaa0000000033333333aaaaaaaa000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000033333333aaaaaaaa0000000033333333aaaaaaaa000000000000000000000000
__label__
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
33b3b3b3b3b3bb3b3b3b3bb33b3b3b3b3b33b3b3b3b3b333b3b3b3b3bb3b3b3b3b3bb3bb3b3b3b33b3b3b3b3b33b3b3bb3b3bb3b3b3b3b3bb3b3b3b3bb33b3b3
b33b33b3bb3bb3b33b33b3bb3bb3b33b33b3bb3bb3b33b333b33b3bb3bb3b33b33b3bb3bb3b33b33b3bb3bb3b33b33b3bb3bb3b3b3bb3b33b33b3bb3b33b33b3
3b3bb3b3bb3b3b33b3b3bb3b3bb3b3b33b33b33b3b33b3b3bb3b3b33b3b33b3b3bb3b3b33b3b33b3b3bb3b3b33b3b33b3b3bb3b3b33b3b33b3b3bb3b3b3b3b3b
3bb3bb33b33bb3bb33b33bb3bb3bb33b33bb3bb33b33bb3bb33b33bb3bb3bb33b33bb3bb33b33bb3bb33b33b33bb3bb33b3b33b33bb3bb33b33bb3bb3bb33b33
b33bb33bb33bb33bb333bb33bb33bb33bb3b33bb33bbb33bb33bb33bb33bb33bb33bbb33bb33bb33bb33bb33bb33bbb33bb33bb33bb33bb33bb333bb33bb33bb
33bbb33bbb33bbb333bb333bb333bb333bb333bb333bb333bb333bb333bb333bb333bb333bb333bbb33bbb33bbb33bbb33bbb33bbb33bbb33bbb33bbb33bbb33
333bbb333bbb333bbb333bbb333bbb333bbb333bbb333bbb333bbb333bb333bbb333bbb333bbb333bbb333bbb333bbb333bbb333bbb333bbb333bbb333bb333b
b333bbbb333bbbb333bbb3333bbb333bbbb333bbbb333bbb3333bbb3333bbb333bbbb333bbbb333bbb3333bbb333bbbb333bbbb333bbb3333bbb3333bb3bbb33
3333bbbb3333bbbb333bbbb3333bbbb3333bbbb333bbbb3333bbbb3333bbbb333bbbb3333bbbb3333bbb3333bbbb3333bbbb3333bbb3333bbbb3333bbbb3333b
b33333bbbb3333bbbb33333bbbb3333bbbbb3333bbbb3333bbbbb3333bbbb3333bbbbb3333bbbb3333bbbbb3333bbbb3333bbbbb3333bbbb3333bbbbb3333bbb
3333bbbbb33333bbbb33333bbbbb3333bbbbb33333bbbbb3333bbbbb33333bbbb33333bbbbb3333bbbbb33333bbbbb3333bbbbb33333bbbb33333bbbbb3333bb
bbbb33333bbbbb33333bbbbb33333bbbbbb33333bbbbb33333bb333bbbbb33333bbbbb333333bbbbb33333bbbbb33333bbbbb33333bbbbbb33333bbbbb33333b
bbbb333333bbbbb333333bbbbb333333bbbbbb33333bbbbbb33333bbbbbb33bbb333333bbbbb333333bbbbbb33333bbbbbb33333bbbbbb33333bbbbbb333333b
bbbbb333bbb333333bbbbbb333333bbbbbb333333bbbbbb333333bbbbbb333333bbbbbb333333bbbbbb333333bbbbbb333333bbbbbb333333bbbbbb333333bbb
b333333bbbbbbb333333bbbbbbb333333bbbbbbb333333bbbbbb3333333bbbbbb3333333bbbbbb3333333bbbbbb3333333bbbbbb333333bbbbbbb333333bbbbb
bbb3333333bbbbbbb3333333bbbbbbb3333333bbbbbbb3333333bbbbbb3333333bbbbbbb3333333bbbbbbb3333333bbbbbbb3333333bbbbbb3333333bbbbbbb3
3333333bbbbbbb3333333bbbbbbbb3333333bbbbbbb33333333bbbbbbb3333333bbbbbbbb3333333bbbbbbb33333333bbbbbbb3333333bbbbbbb33333333bbbb
bbb33333333bbbbbbbb33333333bbbbbbb33333333bbbbbbbb33333333bbbbbbb33333333bbbbbbbb33333333bbbbbbb33333333bbbbbbbb33333333bbbbbbb3
33333333bbbbbbbb33333333bbbbbbbbb33333333bbbbbbbb33333333bbbbbb0003333333bbbbbbbbb33333333bbbbbbbb33333333bbbbbbbb333333333bbbbb
bbbbb333333333bbbbbbbb333333333bbbbbbbb333333333bbbbbbbbb33333033b0bbbbbbb333333333bbbbbbbb333333333bbbbbbbb333333333bbbbbbbbb33
bb333333333bbbbbbbbb333333333bbbbbbbbb333333333bbbbbbbbb333333033b0bbbbbbb333333333bbbbbbbbb333333333bbbbbbbbbb333333333bbbbbbbb
bbbbbbbb3333333333bbbbbbbbb3333333333bbbbbbbbb333333333b333333033b0bbbbbbbb333333333bbbbbbbbbb333333333bbbbbbbbbb333333333bbbbbb
bbbbbb3333333333bbbbbbbbbb333333333bbbbbbbbbb3333333333bbbbbbbb000333333333bbbbbbbbbb3333333333bbbbbbbbbb3333333333bbbbbbbbbb333
bbb3333333333bbbbbbbbbbb3333333333bbbbbbbbbbb3333333333bbbbbbbbbb33333333333bbbbbbbbbb3333333333bbbbbbbbbbb3333333333bbbbbbbbbb3
b3333333333bbbbbbbbbbb33333333333bbbbbbbbbbb3333333333bbbbbbbbbbb33333333333bbbbbbbbbbb33333333333bbbbbbbbbb33333333333bbbbbbbbb
bbbbbbbbb33333333333bbbbbbbbbbbb33333333333bbbbbbbbbbb33333333333bbbbbbbbbbb333333333333bbbbbbbbbbb33333333333bbbbbbbbbbb3333333
bbbbbbb333333333333bbbbbbbbbbb333333333333bbbbbbbbbbbb33333333333bbbbbbbbbbbb333333333333bbbbbbbbbbb333333333333bbbbbbbbbbbb3333
bbbbb333333333333bbbbbbbbbbbb333333333333bbbbbbbbbbbb333333333333bbbbbbbbbbbb333333333333bbbbbbbbbbbbb333333333333bbbbbbbbbbbb33
bbb333333333333bbbbbbbbbbbbb333333333333bbbbbbbbbbbbb333333333333bbbbbbbbbbbbb333333333333bbbbbbbbbbbbb333333333333bbbbbbbbbbbbb
3333333333333bbbbbbbbbbbbb3333333333333bbbbbbbbbbbbb3333333333333bbbbbbbbbbbbb3333333333333bbbbbbbbbbbbb3333333333333bbbbbbbbbbb
bbbbbbbbbbbb3333333333333bbbbbbbbbbbbb33333333333333bbbbbbbbbbbbb33333333333333bbbbbbbbbbbbb3333333333333bbbbbbbbbbbbbb3333bbbbb
bbbbbbbbbb33333333333333bbbbbbbbbbbbbb3333333333333bbbbbbbbbbbbbb33333333333333bbbbbbbbbbbbbb33333333333333bbbbbbbbbbbbbb3333333
bbbbbbbb33333333333333bbbbbbbbbbbbbbb33333333333333bbbbbbbbbbbbbb33333333333333bbbbbbbbbbbbbbb33333333333333bbbbbbbbbbbbbb333333
bbbbbb333333333333333bbbbbbbbbbbbbbb33333333333333bbbbbbbbbbbbbbb333333333333333bbbbbbbbbbbbbbb33333333333333bbbbbbbbbbbbbbb3333
bbbbb333333333333333bbbbbbbbbbbbbbb333333333333333bbbbbbbbbbbbbbb333333333333333bbbbbbbbbbbbbbb3333333333333333bbbbbbbbbbbbbbb33
bbb333333333333333bbbbbbbbbbbbbbbb3333333333333333bbbbbbbbbbbbbbb3333333333333333bbbbbbbbbbbbbbb3333333333333333bbbbbbbbbbbbbbb3
b3333333333333333bbbbbbbbbbbbbbbb3333333333333333bbbbbbbbbbbbbbbb3333333333333333bbbbbbbbbbbbbbbb3333333333333333bbbbbbbbbbbbbbb
bbbbbbbbbbbbbbbb3333333333333333bbbbbbbbbbbbbbbbb333bbbbbbbbbbbbb33333333333333333bbbbbbbbbbbbbbbb3333333333333333bbbbbbbbbbbbbb
bbbbbbbbbbbbbb33333333333333333bbbbbbbbbbbbbbbbb33333333333333333bbbbbbbbbbbbbbbbb33333333333333333bbbbbbbbbbbbbbbbb333333333333
bbbbbbbbbbbbb33333333333333333bbbbbbbbbbbbbbbbbb33333333333333333bbbbbbbbbbbbbbbbb333333333333333333bbbbbbbbbbbbbbbbb33333333333
bbbbbbbbbbbb333333333333333333bbbbbbbbbbbbbbbbb333333333333333333bbbbbbbbbbbbbbbbbb333333333333333333bbbbbbbbbbbbbbbbb3333333333
bbbbbbbbbbb333333333333333333bbbbbbbbbbbbbbbbbb333333333333333333bbbbbbbbbbbbbbbbbb333333333333333333bbbbbbbbbbbbbbbbbbb33333333
bbbbbbbbb3333333333333333333bbbbbbbbbbbbbbbbbb3333333333333333333bbbbbbbbbbbbbbbbbbb333333333333333333bbbbbbbbbbbbbbbbbbb3333333
bbbbbbbb3333333333333333333bbbbbbbbbbbbbbbbbbb3333333333333333333bbbbbbbbbbbbbbbbbbb3333333333333333333bbbbbbbbbbbbbbbbbbb333333
bbbbbbb3333333333333333333bbbbbbbbbbbbbbbbbbbb3333333333333333333bbbbbbbbbbbbbbbbbbbb3333333333333333333bbbbbbbbbbbbbbbbbbbb3333
bbbbb33333333333333333333bbbbbbbbbbbbbbbbbbbb33333333333333333333bbbbbbbbbbbbbbbbbbbb33333333333333333333bbbbbbbbbbbbbbbbbbbb333
bbbb33333333333333333333bbbbbbbbbbbbbbbbbbbbb33333333333333333333bbbbbbbbbbbbbbbbbbbb333333333333333333333bbbbbbbbbbbbbbbbbbbb33
bbb33333333333333333333bbbbbbbbbbbbbbbbbbbbb333333333333333333333bbbbbbbbbbbbbbbbbbbbb333333333333333333333bbbbbbbbbbbbbbbbbbbb3
b3333333333333333333333bbbbbbbbbbbbbbbbbbbbb333333333333333333333bbbbbbbbbbbbbbbbbbbbb3333333333333333333333bbbbbbbbbbbbbbbbbbbb
bbbbbbbbbbbbbbbbbbbbbb3333bbbbbbbbbbbbbbbbb3333333333333333333333bbbbbbbbbbbbbbbbbbbbbb333333333333333333333bbbbbbbbbbbbbbbbbbbb
bbbbbbbbbbbbbbbbbbbbb3333333333333333333333bbbbbbbbbbbbbbbbbbbbbb3333333333333333333333bbbbbbbbbbbbbbbbbbbbbb33333333333333333bb
bbbbbbbbbbbbbbbbbbbb3333333333333333333333bbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333bbbbbbbbbbbbbbbbbbbbbb333333333333333333
bbbbbbbbbbbbbbbbbbb33333333333333333333333bbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333bbbbbbbbbbbbbbbbbbbbbbb33333333333333333
bbbbbbbbbbbbbbbbbb333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbb3333333333333333
bbbbbbbbbbbbbbbbb333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbb333333333333333
bbbbbbbbbbbbbbbb3333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbb33333333333333
bbbbbbbbbbbbbbbb333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbb3333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbb33333333333333
bbbbbbbbbbbbbbb3333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbb3333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbb3333333333333
bbbbbbbbbbbbbb3333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbb333333333333
bbbbbbbbbbbbb33333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbb33333333333
bbbbbbbbbbbb333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbb3333333333
bbbbbbbbbbb333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbb333333333
bbbbbbbbbb3333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbb33333333
bbbbbbbbb3333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbb3333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbb33333333
bbbbbbbbb3333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbb3333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbb3333333
bbbbbbbb3333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbb333333
bbbbbbb33333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbb33333
bbbbbb33333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbb3333
bbbbb333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbb333
bbbb3333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb33
bbb3333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb3333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb3
bb33333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb3333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb3
bb3333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
3bbbbbbbbbbbbbbbbbbbbbbbbbbbbb333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb3bbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333333333
bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb3333333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333333333
bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb3333333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333333333
bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb3333333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333333333
bbbbbbbbbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb3333333333333333333333333333
bbbbbbbbbbbbbbbbbbbbbbbbbbbbbb33333333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb3333333333333333333333333333
bbbbbbbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb3333333333333333333333333333
bbbbbbbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333333
bbbbbbbbbbbbbbbbbbbbbbbbbbbb3333333333333333333333333333333333333bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb333333333333333333333333333

__map__
0809000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
1819000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
